//////////////////////////////////////////////////////////////////////////////
//
// "Embedded C Programming: Techniques and Applications of C and PIC MCUs"
// by Mark Siegusmund
//
// Exercise 12-1 #3
//
// User enters a command and the program responds appropriatly. the allowed 
// commands are as follows:
// LED RED ON           LED RED OFF          LED GREEN ON
// LED GREEN OFF        LED YELLOW ON        LED YELLOW OFF
// READ location (location is a hex number, program responds with the data at
//                that location in RAM).
// WAIT PRESS (wait for button press)        DELAY n (waits n milliseconds)
//
//////////////////////////////////////////////////////////////////////////////

#include <e3.h>
#include <string.h>

#define PB1 PIN_C1
#define PB2 PIN_C2

#define SIZE 15

#define RED PIN_C3
#define YELLOW PIN_C4
#define GREEN PIN_C6

#define ON 1
#define OFF 0

void led(int16 color, int1 state);
void read_ram(int8 addr);
void wait(void);
void delay(int8 time);
char * string_to_upper(char * pString);

void main(void) 
{
   char input[SIZE];
   
   for(;;) {
      input[0] = '\0';
      printf ( "Ready for string\r\n\r\n" );
      
      get_string_usb ( input, SIZE );
   
      string_to_upper(input);
   
      if(!strcmp(input, "LED RED ON"))
         led(RED, ON);
      else if(!strcmp(input, "LED RED OFF"))
         led(RED, OFF);
      else if(!strcmp(input, "LED YELLOW ON"))
         led(YELLOW, ON);
      else if(!strcmp(input, "LED YELLOW OFF"))
         led(YELLOW, OFF);
      else if(!strcmp(input, "LED GREEN ON"))
         led(GREEN, ON);
      else if(!strcmp(input, "LED GREEN OFF"))
         led(GREEN, OFF);
      else if(!strcmp(input, "WAIT PRESS"))
         wait();
      else if(strstr(input, "READ ")) {
         read_ram(strtol(&input[5], NULL, 16));
      }
      else if(strstr(input, "DELAY ")) {
         delay(atol(&input[6]));
      }
      else
         printf("\r\n\r\nUnknown Command, Try Again");
      printf("\r\n");
   }
   
   for(;;);
}

char * string_to_upper(char * pString)
{
   char *ret;
   char c;
   
   ret = pString;
   
   for(;;)
   {
      c = *pString;
      if (!c)
         break;
      *pString++ = toupper(c);
   }
   
   return(ret);
}

void led(int16 color, int1 state) {
   if(color == RED) {
      if(state)
         output_high(RED);
      else
         output_low(RED);
   }
   else if(color == YELLOW) {
      if(state)
         output_high(YELLOW);
      else
         output_low(YELLOW);
   }
   else if(color == GREEN) {
      if(state)
         output_high(GREEN);
      else
         output_low(GREEN);
   }
   else
      printf("\r\n\r\nIT WASN'T ME!!!\r\n\r\n");
}

void read_ram(int8 addr) {
   int8 * ptr = addr;
   printf("\r\n%u is at %u\r\n", *ptr, (int)ptr);
}

void wait(void) {
   while(input(PB1) && input(PB2));
   delay(100);
   while(!(input(PB1) && input(PB2)));
}

void delay(int8 time) {
   delay_ms(time);
}
